@@ -1,13 +1,17 @@
-from adc import auth
-from . import configure
-import os
-import getpass
-import logging
+from collections.abc import Mapping
 import csv
 import errno
+import getpass
+import logging
+import os
 import stat
 import toml
-from collections.abc import Mapping
+
+from adc import auth
+
+from . import configure
+from . import cli
+
 
 SASLMethod = auth.SASLMethod
 
@@ -48,18 +52,52 @@ class Auth(auth.SASLAuth):
 
     @property
     def username(self):
+        """The username for this credential"""
         return self._username
 
+    @property
+    def password(self):
+        """The password for this credential"""
+        return self._config["sasl.password"]
+
     @property
     def hostname(self):
+        """The hostname with which this creential is associated,
+            or the empty string if the credential did not contain this information
+        """
         return self._hostname
 
+    @property
+    def mechanism(self):
+        """The authentication mechanism to use"""
+        return self._config["sasl.mechanism"]
+
+    @property
+    def protocol(self):
+        """The communication protocol to use"""
+        return self._config["security.protocol"]
+
+    @property
+    def ssl(self):
+        """Whether communication should be secured with SSL"""
+        return self.protocol != "SASL_PLAINTEXT"
+
+    @property
+    def ssl_ca_location(self):
+        """The location of the Certfificate Authority data used for SSL,
+            or None if SSL is not enabled
+        """
+        if "ssl.ca.location" not in self._config:
+            return None
+        return self._config["ssl.ca.location"]
+
     def __eq__(self, other):
         return (self._username == other._username
-                and self._config["sasl.password"] == other._config["sasl.password"]
+                and self.password == other.password
                 and self.hostname == other.hostname
-                and self._config["security.protocol"] == other._config["security.protocol"]
-                and self._config["sasl.mechanism"] == other._config["sasl.mechanism"])
+                and self.mechanism == other.mechanism
+                and self.protocol == other.protocol
+                and self.ssl_ca_location == other.ssl_ca_location)
 
 
 def load_auth(config_file=None):
@@ -288,6 +326,7 @@ def read_new_credential(csv_file=None):
                       empty.
 
     """
+    options = {}
     if csv_file is None:
         logger.info("Generating configuration with user-specified username + password")
         username = input("Username: ")
@@ -305,9 +344,15 @@ def read_new_credential(csv_file=None):
                 username = cred["username"]
                 password = cred["password"]
                 hostname = cred["hostname"] if "hostname" in cred else ""
+                if "mechanism" in cred:
+                    options["method"] = cred["mechanism"].replace("-", "_")
+                if "protocol" in cred:
+                    options["ssl"] = cred["protocol"] != "SASL_PLAINTEXT"
+                if "ssl_ca_location" in cred:
+                    options["ssl_ca_location"] = cred["ssl_ca_location"]
         else:
             raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), csv_file)
-    return Auth(username, password, hostname)
+    return Auth(username, password, hostname, **options)
 
 
 def write_auth_data(config_file, credentials):
@@ -322,9 +367,13 @@ def write_auth_data(config_file, credentials):
     """
     cred_list = []
     for cred in credentials:
-        cred_dict = {"username": cred.username, "password": cred._config["sasl.password"]}
+        cred_dict = {"username": cred.username, "password": cred.password,
+                     "protocol": cred.protocol, "mechanism": cred.mechanism}
         if len(cred.hostname) > 0:
             cred_dict["hostname"] = cred.hostname
+        if cred.ssl_ca_location is not None:
+            cred_dict["ssl_ca_location"] = cred.ssl_ca_location
+
         cred_list.append(cred_dict)
 
     os.makedirs(os.path.dirname(config_file), exist_ok=True)
@@ -381,7 +430,7 @@ def add_credential(args):
                 conflicting_cred_idx = idx
     if conflicting_cred_idx is not None:
         if args.force:
-            creds[idx] = new_cred
+            creds[conflicting_cred_idx] = new_cred
         else:
             logger.error("Credential already exists; overwrite with --force")
             return
@@ -474,6 +523,8 @@ def delete_credential(name: str):
 
 
 def _add_parser_args(parser):
+    cli.add_logging_opts(parser)
+
     subparser = parser.add_subparsers(title="commands", metavar="<command>", dest="command")
     subparser.required = True
 
@@ -485,6 +536,9 @@ def _add_parser_args(parser):
                                            "specified either via a CSV file or interactively")
     add_cred_parser.add_argument("cred_file", type=str, default=None, nargs='?',
                                  help="Import credentials from CSV file")
+    add_cred_parser.add_argument("--force", action="store_true", default=False,
+                                 help="If set, overwrite any existing credential when a new one "
+                                 "with the same username and hostname is added")
 
     del_cred_parser = subparser.add_parser("remove", help="Delete a stored credential")
     del_cred_parser.add_argument("name", type=str,
@@ -497,9 +551,7 @@ def _main(args):
     """Authentication configuration utilities.
 
     """
-    logging.basicConfig(
-        level=logging.INFO, format="%(asctime)s | %(name)s : %(levelname)s : %(message)s",
-    )
+    cli.set_up_logger(args)
 
     if args.command == "locate":
         print(configure.get_config_path("auth"))
