@@ -199,25 +199,29 @@ class MockBroker:
         self._messages = defaultdict(list)
         self._offsets = defaultdict(dict)
 
-    def write(self, topic, msg):
-        self._messages[topic].append(msg)
+    def write(self, topic, msg, headers=None):
+        self._messages[topic].append((msg, headers))
 
-    def has_message(self, topic, message):
-        return message in self._messages[topic]
+    def has_message(self, topic, message, headers=None):
+        return (message, headers) in self._messages[topic]
 
-    def read(self, topic, groupid, start_at=StartPosition.EARLIEST, **kwargs):
-        if topic not in self._offsets or groupid not in self._offsets[topic]:
-            if start_at == StartPosition.EARLIEST:
-                self._offsets[topic][groupid] = 0
-            else:
-                self._offsets[topic][groupid] = len(self._messages[topic]) - 1
+    def read(self, topics, groupid, start_at=StartPosition.EARLIEST, **kwargs):
+        if isinstance(topics, str):
+            topics = [topics]
 
-        try:
-            offset = self._offsets[topic][groupid]
-            self._offsets[topic][groupid] += 1
-            yield from self._messages[topic][offset:]
-        except IndexError:
-            pass
+        for topic in topics:
+            if topic not in self._offsets or groupid not in self._offsets[topic]:
+                if start_at == StartPosition.EARLIEST:
+                    self._offsets[topic][groupid] = 0
+                else:
+                    self._offsets[topic][groupid] = len(self._messages[topic]) - 1
+
+            try:
+                offset = self._offsets[topic][groupid]
+                self._offsets[topic][groupid] += 1
+                yield from self._messages[topic][offset:]
+            except IndexError:
+                pass
 
 
 @pytest.fixture(scope="session")
@@ -233,8 +237,8 @@ def mock_producer():
                 self.broker = broker
                 self.topic = topic
 
-            def write(self, msg):
-                self.broker.write(self.topic, msg)
+            def write(self, msg, headers=None, delivery_callback=None):
+                self.broker.write(self.topic, msg, headers)
 
             def close(self):
                 pass
@@ -253,28 +257,34 @@ def mock_producer():
 
 @pytest.fixture(scope="session")
 def mock_consumer():
-    def _mock_consumer(mock_broker, topic, group_id, start_at=StartPosition.EARLIEST):
+    def _mock_consumer(mock_broker, topics, group_id, start_at=StartPosition.EARLIEST):
         class ConsumerBrokerWrapper:
-            def __init__(self, broker, topic, group_id, start_at):
+            def __init__(self, broker, topics, group_id, start_at):
+                if isinstance(topics, str):
+                    topics = [topics]
                 self.broker = broker
-                self.topic = topic
+                self.topics = topics
                 self.group_id = group_id
                 self.start_at = start_at
 
-            def subscribe(self, topic):
-                # TODO: Support multiple topics?
-                assert topic == self.topic
+            def subscribe(self, topics):
+                for topic in topics:
+                    assert topic in set(self.topics)
 
             def stream(self, *args, **kwargs):
                 class Message:
-                    def __init__(self, value):
+                    def __init__(self, value, headers=None):
                         self._value = value
+                        self._headers = headers
 
                     def value(self):
                         return self._value
 
-                for message in self.broker.read(self.topic, self.group_id, self.start_at, **kwargs):
-                    yield Message(message)
+                    def headers(self):
+                        return self._headers
+
+                for message in self.broker.read(self.topics, self.group_id, self.start_at, **kwargs):
+                    yield Message(message[0], message[1])
 
             def close(self):
                 pass
@@ -285,7 +295,7 @@ def mock_consumer():
             def __exit__(self, *exc):
                 pass
 
-        consumer = ConsumerBrokerWrapper(mock_broker, topic, group_id, start_at)
+        consumer = ConsumerBrokerWrapper(mock_broker, topics, group_id, start_at)
         return consumer
 
     return _mock_consumer
@@ -299,6 +309,7 @@ def mock_kafka_message():
     message.offset.return_value = 0
     message.timestamp.return_value = (0, 1234567890)
     message.key.return_value = "test-key"
+    message.headers.return_value = [("a header", "a value"), ("another header", "other value")]
     return message
 
 
