@@ -17,11 +17,53 @@ def check_credential_file(config_path, cred):
     cf = open(config_path, "r")
     config_file_text = cf.read()
     assert cred.username in config_file_text
-    assert cred._config["sasl.password"] in config_file_text
+    assert cred.password in config_file_text
     if len(cred.hostname) > 0:
         assert cred.hostname in config_file_text
 
 
+def test_auth_username():
+    a = auth.Auth("foo", "bar")
+    assert a.username == "foo"
+
+
+def test_auth_password():
+    a = auth.Auth("foo", "bar")
+    assert a.password == "bar"
+
+
+def test_auth_hostname():
+    a = auth.Auth("foo", "bar")  # use default hostname
+    assert a.hostname == ""
+    a = auth.Auth("foo", "bar", "example.com")
+    assert a.hostname == "example.com"
+
+
+def test_auth_mechanism():
+    a = auth.Auth("foo", "bar")  # use default mechanism
+    assert a.mechanism == str(auth.SASLMethod.SCRAM_SHA_512)
+    a = auth.Auth("foo", "bar", method=auth.SASLMethod.SCRAM_SHA_256)
+    assert a.mechanism == str(auth.SASLMethod.SCRAM_SHA_256)
+
+
+def test_auth_protocol():
+    a = auth.Auth("foo", "bar")  # use default security
+    assert a.ssl
+    assert a.protocol == "SASL_SSL"
+    a = auth.Auth("foo", "bar", ssl=False)
+    assert not a.ssl
+    assert a.protocol == "SASL_PLAINTEXT"
+
+
+def test_auth_ca_location():
+    a = auth.Auth("foo", "bar", ssl=False)
+    assert not a.ssl
+    assert a.ssl_ca_location is None
+    a = auth.Auth("foo", "bar", ssl_ca_location="foo/bar")
+    assert a.ssl
+    assert a.ssl_ca_location == "foo/bar"
+
+
 def test_load_auth_legacy(legacy_auth_config, tmpdir):
     with temp_config(tmpdir, legacy_auth_config) as config_dir, \
             temp_environ(XDG_CONFIG_HOME=config_dir):
@@ -169,10 +211,10 @@ def test_load_auth_muliple_creds(tmpdir):
         creds = auth.load_auth()
         assert len(creds) == 2
         assert creds[0].username == "user1"
-        assert creds[0]._config["sasl.password"] == "pass1"
+        assert creds[0].password == "pass1"
         assert creds[0].hostname == "host1"
         assert creds[1].username == "user2"
-        assert creds[1]._config["sasl.password"] == "pass2"
+        assert creds[1].password == "pass2"
         assert creds[1].hostname == "host2"
 
 
@@ -397,7 +439,7 @@ def test_read_new_credential_csv(tmpdir):
         f.write("user,pass")
     new_cred = auth.read_new_credential(csv_file)
     assert new_cred.username == "user"
-    assert new_cred._config["sasl.password"] == "pass"
+    assert new_cred.password == "pass"
     assert new_cred.hostname == ""
 
     # read from a csv file with a hostname
@@ -406,9 +448,37 @@ def test_read_new_credential_csv(tmpdir):
         f.write("user2,pass2,example.com")
     new_cred = auth.read_new_credential(csv_file)
     assert new_cred.username == "user2"
-    assert new_cred._config["sasl.password"] == "pass2"
+    assert new_cred.password == "pass2"
     assert new_cred.hostname == "example.com"
 
+    # read from a csv file with a mechanism
+    with open(csv_file, "w") as f:
+        f.write("username,password,mechanism\n")
+        f.write("user3,pass3,SCRAM-SHA-256")
+    new_cred = auth.read_new_credential(csv_file)
+    assert new_cred.username == "user3"
+    assert new_cred.password == "pass3"
+    assert new_cred.mechanism == "SCRAM_SHA_256"
+
+    # read from a csv file with a mechanism
+    with open(csv_file, "w") as f:
+        f.write("username,password,protocol\n")
+        f.write("user4,pass4,SASL_PLAINTEXT")
+    new_cred = auth.read_new_credential(csv_file)
+    assert new_cred.username == "user4"
+    assert new_cred.password == "pass4"
+    assert not new_cred.ssl
+    assert new_cred.protocol == "SASL_PLAINTEXT"
+
+    # read from a csv file with an SSL CA data location
+    with open(csv_file, "w") as f:
+        f.write("username,password,ssl_ca_location\n")
+        f.write("user5,pass5,foo/bar")
+    new_cred = auth.read_new_credential(csv_file)
+    assert new_cred.username == "user5"
+    assert new_cred.password == "pass5"
+    assert new_cred.ssl_ca_location == "foo/bar"
+
 
 def test_read_new_credential_csv_malformed(tmpdir):
     csv_file = tmpdir + "/cred.csv"
@@ -440,7 +510,7 @@ def test_read_new_credential_interactive(tmpdir):
                 patch("hop.auth.input", MagicMock(side_effect=[username, hostname])):
             new_cred = auth.read_new_credential()
             assert new_cred.username == username
-            assert new_cred._config["sasl.password"] == password
+            assert new_cred.password == password
             assert new_cred.hostname == hostname
 
 
@@ -464,6 +534,13 @@ def test_read_new_credential_interactive_invalid(tmpdir):
     assert err.value.args[0] == "Password may not be empty"
 
 
+def credential_write_read_round_trip(orig_cred, file_path):
+    auth.write_auth_data(file_path, [orig_cred])
+    read_creds = auth.load_auth(file_path)
+    assert len(read_creds) == 1
+    assert read_creds[0] == orig_cred
+
+
 def test_write_config_data(tmpdir):
     config_file = tmpdir + "/config"
     username = "scimma"
@@ -471,6 +548,15 @@ def test_write_config_data(tmpdir):
     auth.write_auth_data(config_file, [auth.Auth(username, password)])
     check_credential_file(config_file, auth.Auth(username, password))
 
+    credential_write_read_round_trip(auth.Auth(username, password), config_file)
+    credential_write_read_round_trip(auth.Auth(username, password, host="example.com"), config_file)
+    credential_write_read_round_trip(auth.Auth(username, password, ssl=False), config_file)
+    credential_write_read_round_trip(auth.Auth(username, password,
+                                               method=auth.SASLMethod.SCRAM_SHA_256),
+                                     config_file)
+    credential_write_read_round_trip(auth.Auth(username, password, ssl_ca_location="ca.cert"),
+                                     config_file)
+
 
 def test_list_credentials(tmpdir, capsys):
     with patch("hop.auth.load_auth", MagicMock(return_value=[])):
@@ -492,7 +578,7 @@ def test_list_credentials(tmpdir, capsys):
         auth.list_credentials()
         captured = capsys.readouterr()
         assert short_cred.username in captured.out
-        assert short_cred._config["sasl.password"] not in captured.out
+        assert short_cred.password not in captured.out
         assert "for" not in captured.out
 
     with patch("hop.auth.load_auth", MagicMock(return_value=[short_cred, long_cred])):
@@ -500,7 +586,7 @@ def test_list_credentials(tmpdir, capsys):
         captured = capsys.readouterr()
         assert short_cred.username in captured.out
         assert long_cred.username in captured.out
-        assert long_cred._config["sasl.password"] not in captured.out
+        assert long_cred.password not in captured.out
         assert long_cred.hostname in captured.out
 
 
@@ -586,10 +672,11 @@ def test_add_credential_conflict_no_host(tmpdir, caplog):
 def test_add_credential_conflict_with_host(tmpdir, caplog):
     old_cred = auth.Auth("username", "password", "example.com")
     new_cred = auth.Auth("username", "other_pass", "example.com")
+    unrelated_cred = auth.Auth("username", "unrelated_pass", "example.org")
 
     with temp_environ(HOME=str(tmpdir)), \
             patch("hop.auth.read_new_credential", MagicMock(return_value=new_cred)):
-        auth.write_auth_data(configure.get_config_path("auth"), [old_cred])
+        auth.write_auth_data(configure.get_config_path("auth"), [old_cred, unrelated_cred])
         args = MagicMock()
         args.cred_file = None
         args.force = False
@@ -602,6 +689,9 @@ def test_add_credential_conflict_with_host(tmpdir, caplog):
         auth.add_credential(args)
         # with the force option, the old credential should be overwritten
         check_credential_file(configure.get_config_path("auth"), new_cred)
+        # also check that unrelated credentials haven't been removed
+        creds = auth.load_auth(configure.get_config_path("auth"))
+        assert unrelated_cred in creds
 
 
 delete_input_creds = [
