@@ -1,3 +1,4 @@
+from collections import Counter
 from dataclasses import fields
 import json
 import logging
@@ -85,7 +86,7 @@ def test_stream_read(circular_msg):
     fake_message.value = MagicMock(return_value=json.dumps(message_data).encode("utf-8"))
     mock_instance = MagicMock()
     mock_instance.stream = MagicMock(return_value=[fake_message])
-    stream = io.Stream(persist=False, start_at=start_at, auth=False)
+    stream = io.Stream(start_at=start_at, until_eos=True, auth=False)
     with patch("hop.io.consumer.Consumer", MagicMock(return_value=mock_instance)):
         broker_url1 = f"kafka://hostname:port/{topic}"
         broker_url2 = f"kafka://{group_id}@hostname:port/{topic}"
@@ -103,18 +104,47 @@ def test_stream_read(circular_msg):
         assert messages == 1
 
 
+def test_stream_read_multiple(circular_msg):
+    group_id = "test-group"
+    topic1 = "gcn1"
+    topic2 = "gcn2"
+    start_at = io.StartPosition.EARLIEST
+
+    message_data = {"format": "circular", "content": circular_msg}
+    topic1_message = MagicMock()
+    topic1_message.value = MagicMock(return_value=json.dumps(message_data).encode("utf-8"))
+    topic1_message.topic = MagicMock(return_value=topic1)
+    topic2_message = MagicMock()
+    topic2_message.value = MagicMock(return_value=json.dumps(message_data).encode("utf-8"))
+    topic2_message.topic = MagicMock(return_value=topic2)
+    mock_instance = MagicMock()
+    mock_instance.stream = MagicMock(return_value=[topic1_message, topic2_message])
+    stream = io.Stream(start_at=start_at, until_eos=True, auth=False)
+    with patch("hop.io.consumer.Consumer", MagicMock(return_value=mock_instance)):
+        broker_url = f"kafka://{group_id}@hostname:port/{topic1},{topic2}"
+
+        messages = Counter()
+        with stream.open(broker_url, "r") as s:
+            for msg, meta in s.read(metadata=True):
+                messages[meta.topic] += 1
+
+        assert messages[topic1] == 1
+        assert messages[topic2] == 1
+
+
 def test_stream_write(circular_msg, circular_text, mock_broker, mock_producer):
     topic = "gcn"
     mock_adc_producer = mock_producer(mock_broker, topic)
     expected_msg = json.dumps(Blob(circular_msg).serialize()).encode("utf-8")
+    headers = {"some header": "some value"}
     with patch("hop.io.producer.Producer", autospec=True, return_value=mock_adc_producer):
 
         broker_url = f"kafka://localhost:port/{topic}"
         auth = Auth("user", "password")
         start_at = io.StartPosition.EARLIEST
-        persist = False
+        until_eos = True
 
-        stream = io.Stream(start_at=start_at, persist=persist, auth=auth)
+        stream = io.Stream(start_at=start_at, until_eos=until_eos, auth=auth)
 
         # verify only 1 topic is allowed in write mode
         with pytest.raises(ValueError):
@@ -126,15 +156,15 @@ def test_stream_write(circular_msg, circular_text, mock_broker, mock_producer):
 
         mock_broker.reset()
         with stream.open(broker_url, "w") as s:
-            s.write(circular_msg)
-            assert mock_broker.has_message(topic, expected_msg)
+            s.write(circular_msg, headers)
+            assert mock_broker.has_message(topic, expected_msg, headers)
 
         # repeat, but with a manual close instead of context management
         mock_broker.reset()
         s = stream.open(broker_url, "w")
-        s.write(circular_msg)
+        s.write(circular_msg, headers)
         s.close()
-        assert mock_broker.has_message(topic, expected_msg)
+        assert mock_broker.has_message(topic, expected_msg, headers)
 
 
 def test_stream_auth(auth_config, tmpdir):
@@ -192,7 +222,7 @@ def test_stream_open(auth_config, tmpdir):
         # an appropriate consumer group name should be derived from the username in the auth
         assert consumer._consumer.conf.group_id.startswith(stream.auth[0].username)
         # the target topic should be subscribed to
-        subscribe.assert_called_once_with("topic")
+        subscribe.assert_called_once_with(["topic"])
 
         # opening a valid URL for writing should succeed
         producer = stream.open("kafka://example.com/topic", "w")
@@ -219,7 +249,7 @@ def test_mark_done(circular_msg):
     mock_message.value = MagicMock(return_value=json.dumps(message_data).encode("utf-8"))
     mock_instance = MagicMock()
     mock_instance.stream = MagicMock(return_value=[mock_message])
-    stream = io.Stream(persist=False, start_at=start_at, auth=False)
+    stream = io.Stream(until_eos=True, start_at=start_at, auth=False)
 
     with patch("hop.io.consumer.Consumer", MagicMock(return_value=mock_instance)):
         with stream.open("kafka://hostname:port/gcn", "r") as s:
@@ -326,6 +356,7 @@ def test_metadata(mock_kafka_message):
     assert metadata.offset == mock_kafka_message.offset()
     assert metadata.timestamp == mock_kafka_message.timestamp()[1]
     assert metadata.key == mock_kafka_message.key()
+    assert metadata.headers == mock_kafka_message.headers()
 
 
 def test_plugin_loading(caplog):
@@ -385,3 +416,136 @@ def test_plugin_loading(caplog):
         assert "VOEVENT" in registered
         assert "CIRCULAR" in registered
         assert "BLOB" in registered
+
+
+def make_mock_listing_consumer(topics=[]):
+    """Create a mock object suitable for replacing confluent_kafka.Consumer
+        which has a list_topics method which acts as if a predetermined set of
+        topics exist.
+    """
+    def get_topics(topic=None):
+        nonlocal topics
+        result = {}
+        if topic is None:
+            for topic in topics:
+                result[topic] = MagicMock()  # don't care much what value is
+                result[topic].error = None  # but it should claim no errors
+        elif topic in topics:
+            result[topic] = MagicMock()
+            result[topic].error = None
+        # wrap in a fake metadata object
+        metadata = MagicMock()
+        metadata.topics = result
+        return metadata
+    consumer = MagicMock()
+    consumer.list_topics = get_topics
+    return MagicMock(return_value=consumer)
+
+
+def test_list_topics():
+    # when there are some topics, they are all listed
+    with patch("confluent_kafka.Consumer", make_mock_listing_consumer(["foo", "bar"])) as Consumer:
+        listing = io.list_topics("kafka://example.com", auth=False)
+        assert(len(listing) == 2)
+        assert("foo" in listing)
+        assert("bar" in listing)
+
+        # when auth=False, no auth related properties are set
+        Consumer.assert_called_once()
+        cons_args = Consumer.call_args[0]
+        assert(len(cons_args) == 1)
+        assert("sasl.username" not in cons_args[0])
+        assert("sasl.password" not in cons_args[0])
+
+    # when there are no topics, the result is empty
+    with patch("confluent_kafka.Consumer", make_mock_listing_consumer([])) as Consumer:
+        listing = io.list_topics("kafka://example.com", auth=False)
+        assert(len(listing) == 0)
+
+    # result topics should be the intersection of ones which exist and which were requested
+    with patch("confluent_kafka.Consumer", make_mock_listing_consumer(["foo", "bar"])) as Consumer:
+        listing = io.list_topics("kafka://example.com/bar,baz", auth=False)
+        assert(len(listing) == 1)
+        assert("bar" in listing)
+
+
+def test_list_topics_too_many_brokers():
+    # If the URL specifies too many brokers, an error is raised
+    with patch("confluent_kafka.Consumer", make_mock_listing_consumer([])):
+        with pytest.raises(ValueError):
+            listing = io.list_topics("kafka://example.com,example.net", auth=False)
+
+
+def test_list_topics_auth(auth_config, tmpdir):
+    # when auth=True, auth related properties should be set
+    with temp_config(tmpdir, auth_config) as config_dir, \
+            temp_environ(XDG_CONFIG_HOME=config_dir), \
+            patch("confluent_kafka.Consumer", make_mock_listing_consumer([])) as Consumer:
+        listing = io.list_topics("kafka://example.com", auth=True)
+
+        # when auth=True, auth related properties should be set
+        Consumer.assert_called_once()
+        cons_args = Consumer.call_args[0]
+        assert(len(cons_args) == 1)
+        assert("sasl.username" in cons_args[0])
+        assert("sasl.password" in cons_args[0])
+        assert(cons_args[0]["sasl.username"] == "username")
+        assert(cons_args[0]["sasl.password"] == "password")
+
+    # when an Auth object is set, it should take precedence over automatic lookup
+    with temp_config(tmpdir, auth_config) as config_dir, \
+            temp_environ(XDG_CONFIG_HOME=config_dir), \
+            patch("confluent_kafka.Consumer", make_mock_listing_consumer([])) as Consumer:
+        auth = Auth("someone_else", "other_password")
+        listing = io.list_topics("kafka://example.com", auth=auth)
+
+        # when auth=Auth(...), auth related properties should be set correctly
+        Consumer.assert_called_once()
+        cons_args = Consumer.call_args[0]
+        assert(len(cons_args) == 1)
+        assert("sasl.username" in cons_args[0])
+        assert("sasl.password" in cons_args[0])
+        assert(cons_args[0]["sasl.username"] == auth.username)
+        assert(cons_args[0]["sasl.password"] == auth.password)
+
+    # when an Auth object is set, it should take precedence over automatic lookup,
+    # even with userinfo in the URL
+    with temp_config(tmpdir, auth_config) as config_dir, \
+            temp_environ(XDG_CONFIG_HOME=config_dir), \
+            patch("confluent_kafka.Consumer", make_mock_listing_consumer([])) as Consumer:
+        auth = Auth("someone_else", "other_password")
+        listing = io.list_topics("kafka://username@example.com", auth=auth)
+
+        # when auth=Auth(...), auth related properties should be set correctly
+        Consumer.assert_called_once()
+        cons_args = Consumer.call_args[0]
+        assert(len(cons_args) == 1)
+        assert("sasl.username" in cons_args[0])
+        assert("sasl.password" in cons_args[0])
+        assert(cons_args[0]["sasl.username"] == auth.username)
+        assert(cons_args[0]["sasl.password"] == auth.password)
+
+    # when auth=True and userinfo, the correct credential should be automatically
+    # looked up
+    multi_cred = """
+        auth = [{
+            username="user1",
+            password="pass1"
+            },
+            {
+            username="user2",
+            password="pass2"
+            }]
+        """
+    with temp_config(tmpdir, multi_cred) as config_dir, \
+            temp_environ(XDG_CONFIG_HOME=config_dir), \
+            patch("confluent_kafka.Consumer", make_mock_listing_consumer([])) as Consumer:
+        listing = io.list_topics("kafka://user2@example.com", auth=True)
+
+        Consumer.assert_called_once()
+        cons_args = Consumer.call_args[0]
+        assert(len(cons_args) == 1)
+        assert("sasl.username" in cons_args[0])
+        assert("sasl.password" in cons_args[0])
+        assert(cons_args[0]["sasl.username"] == "user2")
+        assert(cons_args[0]["sasl.password"] == "pass2")
